/**
 * @file
 * VuoModule implementation.
 *
 * @copyright Copyright © 2012–2013 Kosada Incorporated.
 * This code is proprietary, and may not be redistributed.
 * For more information, see http://vuo.org/license.
 */

#include "VuoModule.hh"

/**
 * Creates a module.
 *
 * @param moduleKey A unique name for this module.
 */
VuoModule::VuoModule(string moduleKey)
{
	this->moduleKey = moduleKey;
}

/**
 * Returns this module's unique name.
 */
string VuoModule::getModuleKey(void)
{
	return moduleKey;
}

/**
 * Returns the default display name for instances of this module, as specified by the .vuonode/.bc implementation.  Needn't be unique.
 *
 * UTF-8.
 *
 * @eg{Less Than}
 *
 * @see VuoModuleDetails
 */
string VuoModule::getDefaultDisplayName(void)
{
	return defaultDisplayName;
}

/**
 * Sets the default display name for instances of this module.  Needn't be unique.
 *
 * UTF-8.
 *
 * @eg{Less Than}
 *
 * @see VuoModuleDetails
 */
void VuoModule::setDefaultDisplayName(string defaultDisplayName)
{
	this->defaultDisplayName = defaultDisplayName;
}

/**
 * Returns the description of this module provided as documentation.
 *
 * @see VuoModuleDetails
 */
string VuoModule::getDescription(void)
{
	return description;
}

/**
 * Sets the description of this module provided as documentation.
 *
 * @see VuoModuleDetails
 */
void VuoModule::setDescription(string description)
{
	this->description = description;
}

/**
 * Returns the module's version, in Semantic Versioning format.
 *
 * @see VuoModuleDetails
 */
string VuoModule::getVersion(void)
{
	return version;
}

/**
 * Sets the module's version, in Semantic Versioning format.
 *
 * @see VuoModuleDetails
 */
void VuoModule::setVersion(string version)
{
	this->version = version;
}

/**
 * Returns a list of the module's keywords.
 *
 * @see VuoModuleDetails
 */
vector<string> VuoModule::getKeywords(void)
{
	return keywords;
}

/**
 * Sets the module's keywords.
 *
 * @see VuoModuleDetails
 */
void VuoModule::setKeywords(vector<string> keywords)
{
	this->keywords = keywords;
}
