/**
 * @file
 * VuoPortClass implementation.
 *
 * @copyright Copyright © 2012–2013 Kosada Incorporated.
 * This code is proprietary, and may not be redistributed.
 * For more information, see http://vuo.org/license.
 */

#include "VuoPortClass.hh"

/**
 * Creates a base port class.
 */
VuoPortClass::VuoPortClass(string name, PortType portType)
	: VuoBase<VuoCompilerNodeArgumentClass,void>("VuoPortClass")
{
	this->name = name;
	this->portType = portType;
	this->conductivity = alwaysConducts;
}

/**
 * Returns the name of this port class.
 */
string VuoPortClass::getName(void)
{
	return name;
}

/**
 * Returns this port's type.
 */
VuoPortClass::PortType VuoPortClass::getPortType(void)
{
	return portType;
}

/**
 * Returns this port's conductivity.
 */
VuoPortClass::Conductivity VuoPortClass::getConductivity(void)
{
	return conductivity;
}

/**
 * Sets this port's conductivity.
 */
void VuoPortClass::setConductivity(VuoPortClass::Conductivity conductivity)
{
	this->conductivity = conductivity;
}

/**
 * Prints info about this port class, for debugging.
 */
void VuoPortClass::print(void)
{
	printf("VuoPortClass(%p,\"%s\")",this,getName().c_str());
	if (hasCompiler())
		printf(" VuoCompilerPortClass(%p)",getCompiler());
	if (hasRenderer())
		printf(" VuoRendererPortClass(%p)",getRenderer());
	printf("\n");
}
