/**
 * @file
 * VuoWindow interface.
 *
 * @copyright Copyright © 2012–2013 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#ifdef __cplusplus
extern "C"
{
#endif

#include "node.h"

/**
 * A window containing a text edit widget.
 */
typedef void * VuoWindowText;

/**
 * A window containing an OpenGL view.
 */
typedef void * VuoWindowOpenGL;

VuoWindowText VuoWindowTextCreate(void);
void VuoWindowTextEnableTriggers
(
		VuoWindowText w,
		VuoOutputTrigger(typedLine, VuoText),
		VuoOutputTrigger(typedWord, VuoText),
		VuoOutputTrigger(typedCharacter, VuoText)
);
void VuoWindowTextDisableTriggers(VuoWindowText w);
void VuoWindowTextAppendLine(VuoWindowText w, const char *text);
void VuoWindowTextDestroy(VuoWindowText w);

VuoWindowOpenGL VuoWindowOpenGLCreate
(
		void (*initCallback)(void *),
		void (*resizeCallback)(void *, unsigned int, unsigned int),
		void (*drawCallback)(void *),
		void *context
);
void VuoWindowOpenGLEnableTriggers
(
		VuoWindowOpenGL w,
		VuoOutputTrigger(requestedFrame, VuoFrameRequest),
		VuoOutputTrigger(mouseMoved, VuoPoint2d),
		VuoOutputTrigger(mouseLeftPressed, VuoPoint2d)
);
void VuoWindowOpenGLDisableTriggers(VuoWindowOpenGL w);
void VuoWindowOpenGLRedraw(VuoWindowOpenGL w);
void VuoWindowOpenGLSetAspectRatio(VuoWindowOpenGL w, unsigned int pixelsWide, unsigned int pixelsHigh);
void VuoWindowOpenGLDestroy(VuoWindowOpenGL w);

#ifdef __cplusplus
}
#endif
