﻿/**
 * @file
 * vuo.console.window node implementation.
 *
 * @copyright Copyright © 2012–2013 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#include "node.h"
#include <stdlib.h>
#include <unistd.h>
#include <dispatch/dispatch.h>
#include <errno.h>

#include "VuoWindow.h"

VuoModuleDetails({
					 "name" : "Display Console Window",
					 "description" : "Opens a window that displays console output (text written to the `lineToWrite` input) and accepts text entered by the user (made available on the `typed*` triggers).",
					 "keywords" : [ "text", "string", "print", "log", "write", "read", "input", "output" ],
					 "version" : "1.0.0",
					 "dependencies" : [
						 "VuoWindow"
					 ],
					 "node": {
						 "isInterface" : true
					 }
				 });


struct nodeInstanceData
{
	VuoWindowText *window;
};

struct nodeInstanceData * nodeInstanceInit(void)
{
	struct nodeInstanceData *context = (struct nodeInstanceData *)calloc(1,sizeof(struct nodeInstanceData));
	context->window = VuoWindowTextCreate();
	VuoRegister(context, free);
	return context;
}

void nodeInstanceCallbackStart
(
		VuoInstanceData(struct nodeInstanceData *) context,
		VuoOutputTrigger(typedLine, VuoText),
		VuoOutputTrigger(typedWord, VuoText),
		VuoOutputTrigger(typedCharacter, VuoText)
)
{
	VuoWindowTextEnableTriggers((*context)->window, typedLine, typedWord, typedCharacter);
}

void nodeInstanceEvent
(
		VuoInstanceData(struct nodeInstanceData *) context,
		VuoInputData(VuoText, {"default":""}) lineToWrite,
		VuoInputConductor(VuoPortAlwaysConducts, lineToWrite) lineToWriteEvent
)
{
	if (lineToWriteEvent)
		VuoWindowTextAppendLine((*context)->window, lineToWrite);
}

void nodeInstanceCallbackStop
(
		VuoInstanceData(struct nodeInstanceData *) context
)
{
	VuoWindowTextDisableTriggers((*context)->window);
}

void nodeInstanceFini
(
		VuoInstanceData(struct nodeInstanceData *) context
)
{
	VuoWindowTextDestroy((*context)->window);
}
