﻿/**
 * @file
 * vuo.hold.VuoReal node implementation.
 *
 * @copyright Copyright © 2012–2013 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#include "node.h"

VuoModuleDetails({
					 "name" : "Hold Value",
					 "description" : "This node holds onto the most recent value received through the 'newValue' port and outputs it when an event hits the refresh port. If the 'newValue' port has never received an event, this node instead outputs 'initialValue'.",
					 "keywords" : [ "store", "sample", "feedback" ],
					 "version" : "1.0.0",
					 "node": {
						 "isInterface" : false
					 }
				 });

bool * nodeInstanceInit()
{
	bool *hasNewValueReceivedEvent = (bool *) malloc(sizeof(bool));
	VuoRegister(hasNewValueReceivedEvent, free);
	*hasNewValueReceivedEvent = false;
	return hasNewValueReceivedEvent;
}

void nodeInstanceEvent
(
		VuoInstanceData(bool *) hasNewValueReceivedEvent,
		VuoInputData(VuoReal, {"default":0.0}) initialValue,
		VuoInputConductor(VuoPortNeverConducts,initialValue) initialValueEvent,
		VuoInputData(VuoReal, {"default":0.0}) newValue,
		VuoInputConductor(VuoPortNeverConducts,newValue) newValueEvent,
		VuoOutputData(VuoReal) heldValue
)
{
	if (newValueEvent)
		**hasNewValueReceivedEvent = true;

	*heldValue = (**hasNewValueReceivedEvent ? newValue : initialValue);
}

void nodeInstanceFini(VuoInstanceData(bool *) hasNewValueReceivedEvent)
{
}
