/**
 * @file
 * vuo.math.count.real node implementation.
 *
 * @copyright Copyright © 2012–2013 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#include "node.h"
#include <stdlib.h>

VuoModuleDetails({
					 "name" : "Count",
					 "description" : "Keeps track of a count that starts at 0. The count increases by `increment` when that port is pushed. It decreases by `decrement` when that port is pushed. When `refresh` is pushed, the count stays the same.",
					 "keywords" : [ ],
					 "version" : "1.0.0",
					 "node": {
						 "isInterface" : false
					 }
				 });


VuoReal * nodeInstanceInit()
{
	VuoReal *countState = (VuoReal *) malloc(sizeof(VuoReal));
	VuoRegister(countState, free);
	*countState = 0;
	return countState;
}

void nodeInstanceEvent
(
		VuoInstanceData(VuoReal *) countState,
		VuoInputData(VuoReal, {"default":1}) increment,
		VuoInputConductor(VuoPortAlwaysConducts,increment) incrementEvent,
		VuoInputData(VuoReal, {"default":1}) decrement,
		VuoInputConductor(VuoPortAlwaysConducts,decrement) decrementEvent,
		VuoOutputData(VuoReal) count
)
{
	if (incrementEvent)
		**countState += increment;
	if (decrementEvent)
		**countState -= decrement;
	*count = **countState;
}


void nodeInstanceFini
(
		VuoInstanceData(VuoReal *) countState
)
{
}
