/**
 * @file
 * vuo.midi.receive node implementation.
 *
 * @copyright Copyright © 2012–2013 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#include "node.h"
#include "VuoMidi.h"

VuoModuleDetails({
					 "name" : "Receive MIDI Events",
					 "description" : "Listens for MIDI data.  When data is received, this node fires an event for the corresponding type of MIDI data.  The first available MIDI Input device is used, unless a specific `device` is provided.",
					 "keywords" : [ ],
					 "version" : "1.0.0",
					 "dependencies" : [
						 "VuoMidi"
					 ],
					 "node": {
						 "isInterface" : true
					 }
				 });


VuoMidiIn nodeInstanceInit(
		VuoInputData(VuoMidiDevice, {"default":{"isInput":true}}) device
)
{
	return VuoMidiIn_make(device);
}

void nodeInstanceCallbackStart
(
		VuoInstanceData(VuoMidiIn) context,
		VuoOutputTrigger(receivedNote, VuoMidiNote),
		VuoOutputTrigger(receivedController, VuoMidiController)
//		VuoOutputTrigger(receivedAftertouch, VuoMidiAftertouch),
//		VuoOutputTrigger(receivedCommand, VuoMidiCommand),
//		VuoOutputTrigger(receivedClock, VuoMidiClock),
//		VuoOutputTrigger(receivedTimecode, VuoMidiTimecode),
//		VuoOutputTrigger(receivedSysEx, VuoMidiSysEx)
)
{
	VuoMidiIn_enableTriggers(*context, receivedNote, receivedController);
}

void nodeInstanceEvent
(
		VuoInstanceData(VuoMidiIn) context,
		VuoInputData(VuoMidiDevice, {"default":{"isInput":true}}) device,
		VuoInputConductor(VuoPortNeverConducts, device) deviceEvent
)
{
	if (deviceEvent)
	{
		VuoMidiIn_destroy(*context);
		*context = VuoMidiIn_make(device);
	}
}

void nodeInstanceCallbackStop
(
		VuoInstanceData(VuoMidiIn) context
)
{
	VuoMidiIn_disableTriggers(*context);
}

void nodeInstanceFini
(
		VuoInstanceData(VuoMidiIn) context
)
{
	VuoMidiIn_destroy(*context);
}
