/**
 * @file
 * vuo.midi.send node implementation.
 *
 * @copyright Copyright © 2012–2013 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#include "node.h"
#include "VuoMidi.h"

VuoModuleDetails({
					 "name" : "Send MIDI Event",
					 "description" : "When an event is received on the `note` or `controller` ports, this node outputs the data to a MIDI device.  The first available MIDI output device is used, unless a specific `device` is provided.",
					 "keywords" : [ ],
					 "version" : "1.0.0",
					 "dependencies" : [
						 "VuoMidi"
					 ],
					 "node": {
						 "isInterface" : true
					 }
				 });


VuoMidiOut nodeInstanceInit(
		VuoInputData(VuoMidiDevice, {"default":{"isInput":false}}) device
)
{
	return VuoMidiOut_make(device);
}

void nodeInstanceEvent
(
		VuoInstanceData(VuoMidiOut) context,
		VuoInputData(VuoMidiDevice, {"default":{"isInput":false}}) device,
		VuoInputConductor(VuoPortNeverConducts, device) deviceEvent,
		VuoInputData(VuoMidiNote,"") note,
		VuoInputConductor(VuoPortAlwaysConducts, note) noteEvent,
		VuoInputData(VuoMidiController,"") controller,
		VuoInputConductor(VuoPortAlwaysConducts, controller) controllerEvent
//		VuoInputData(VuoMidiAftertouch,"") aftertouch,
//		VuoInputData(VuoMidiCommand,"") command,
//		VuoInputData(VuoMidiClock,"") clock,
//		VuoInputData(VuoMidiTimecode,"") timecode,
//		VuoInputData(VuoMidiSysEx,"") sysEx
)
{
	if (deviceEvent)
	{
		VuoMidiOut_destroy(*context);
		*context = VuoMidiOut_make(device);
	}

	if (noteEvent)
		VuoMidiOut_sendNote(*context, note);
	if (controllerEvent)
		VuoMidiOut_sendController(*context, controller);
}

void nodeInstanceFini
(
		VuoInstanceData(VuoMidiOut) context
)
{
	VuoMidiOut_destroy(*context);
}
