/**
 * @file
 * vuo.mouse.filter.action node implementation.
 *
 * @copyright Copyright © 2012–2013 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#include "node.h"

VuoModuleDetails({
					 "name" : "Filter Mouse Button Action",
					 "description" : "Blocks the `action` event unless the mouse button action matches all conditions.",
					 "keywords" : [ ],
					 "version" : "1.0.0",
					 "node": {
						 "isInterface" : false
					 }
				 });


void nodeEvent
(
		VuoInputData(VuoMouseButtonAction) action,
		VuoInputConductor(VuoPortMayConduct, action) actionEvent,

		VuoInputData(VuoBoolean, {"default":true}) includeLeftButton,
		VuoInputConductor(VuoPortNeverConducts, includeLeftButton) includeLeftButtonEvent,

		VuoInputData(VuoBoolean, {"default":false}) includeMiddleButton,
		VuoInputConductor(VuoPortNeverConducts, includeMiddleButton) includeMiddleButtonEvent,

		VuoInputData(VuoBoolean, {"default":false}) includeRightButton,
		VuoInputConductor(VuoPortNeverConducts, includeRightButton) includeRightButtonEvent,

		VuoInputData(VuoBoolean, {"default":true}) includePress,
		VuoInputConductor(VuoPortNeverConducts, includePress) includePressEvent,

		VuoInputData(VuoBoolean, {"default":false}) includeRelease,
		VuoInputConductor(VuoPortNeverConducts, includeRelease) includeReleaseEvent,

		VuoInputData(VuoBoolean, {"default":false}) includeSingleClick,
		VuoInputConductor(VuoPortNeverConducts, includeSingleClick) includeSingleClickEvent,

		VuoInputData(VuoBoolean, {"default":false}) includeDoubleClick,
		VuoInputConductor(VuoPortNeverConducts, includeDoubleClick) includeDoubleClickEvent,

		VuoInputData(VuoBoolean, {"default":false}) includeTripleClick,
		VuoInputConductor(VuoPortNeverConducts, includeTripleClick) includeTripleClickEvent,

		VuoOutputData(VuoMouseButtonAction) filteredAction,
		VuoOutputConductor(filteredAction) filteredActionEvent
)
{
	if (action.button == VuoMouseButton_Left && ! includeLeftButton)
		return;

	if (action.button == VuoMouseButton_Middle && ! includeMiddleButton)
		return;

	if (action.button == VuoMouseButton_Right && ! includeRightButton)
		return;

	if (action.type == VuoMouseButtonActionType_Press && ! includePress)
		return;

	if (action.type == VuoMouseButtonActionType_Release && ! includeRelease)
		return;

	if (action.type == VuoMouseButtonActionType_SingleClick && ! includeSingleClick)
		return;

	if (action.type == VuoMouseButtonActionType_DoubleClick && ! includeDoubleClick)
		return;

	if (action.type == VuoMouseButtonActionType_TripleClick && ! includeTripleClick)
		return;

	*filteredAction = action;
	*filteredActionEvent = true;
}
