/**
 * @file
 * compositionForListening implementation.
 *
 * @copyright Copyright © 2012–2013 Kosada Incorporated.
 * This code is proprietary, and may not be redistributed.
 * For more information, see http://vuo.org/license.
 */

#include <dispatch/dispatch.h>
#include <CoreFoundation/CoreFoundation.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include "composition.h"

dispatch_source_t timer = NULL;

int main(int argc, char **argv)
{
	vuoInit(argc, argv);

	{
		dispatch_queue_t queue = dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0);
		timer = dispatch_source_create(DISPATCH_SOURCE_TYPE_TIMER,0,0,queue);
		dispatch_source_set_timer(timer, dispatch_walltime(NULL, NSEC_PER_SEC/4), NSEC_PER_SEC/2, NSEC_PER_SEC/100);
		dispatch_source_set_event_handler(timer, ^{
			{
				zmq_msg_t messages[1];
				const char *nodeIdentifier = "node";
				zmq_msg_init_size(&messages[0], strlen(nodeIdentifier) + 1);
				memcpy(zmq_msg_data(&messages[0]), nodeIdentifier, strlen(nodeIdentifier) + 1);
				vuoTelemetrySend(VuoTelemetryNodeExecutionStarted, messages, 1);
			}

			{
				zmq_msg_t messages[1];
				const char *edgeIdentifier = "edge";
				zmq_msg_init_size(&messages[0], strlen(edgeIdentifier) + 1);
				memcpy(zmq_msg_data(&messages[0]), edgeIdentifier, strlen(edgeIdentifier) + 1);
				vuoTelemetrySend(VuoTelemetryEdgeHit, messages, 1);
			}
		});
		dispatch_resume(timer);
	}

	while (! isStopped)
	{
		_dispatch_main_queue_callback_4CF(0);
		usleep(10000);
	}
	return 0;
}

void nodeInstanceFini(void)
{
	dispatch_source_cancel(timer);
	dispatch_release(timer);
}
