/**
 * @file
 * TestVuoPoint2d implementation.
 *
 * @copyright Copyright © 2012–2013 Kosada Incorporated.
 * This code is proprietary, and may not be redistributed.
 * For more information, see http://vuo.org/license.
 */

extern "C" {
#include "TestVuoTypes.h"
#include "VuoPoint2d.h"
}

// Be able to use this type in QTest::addColumn()
Q_DECLARE_METATYPE(VuoPoint2d);

/**
 * Tests the VuoPoint2d type.
 */
class TestVuoPoint2d : public QObject
{
	Q_OBJECT

private slots:
	void initTestCase()
	{
		VuoHeap_init();
	}

	void testStringConversion_data()
	{
		QTest::addColumn<QString>("initializer");
		QTest::addColumn<VuoPoint2d>("value");
		QTest::addColumn<bool>("testStringFromValue");

		{
			VuoPoint2d p;
			p.x = p.y = 0;
			QTest::newRow("zero") << "{\"x\":0.000000,\"y\":0.000000}" << p << true;
			QTest::newRow("emptystring") << "" << p << false;
			QTest::newRow("partial point 1") << "{\"y\":0}" << p << false;
			QTest::newRow("nonpoint") << "Otto von Bismarck" << p << false;
		}

		{
			VuoPoint2d p;
			p.x = p.y = 1;
			QTest::newRow("one") << "{\"x\":1.000000,\"y\":1.000000}" << p << true;
		}

		{
			VuoPoint2d p;
			p.x = -0.999;
			p.y = 0.42;
			QTest::newRow("different values") << "{\"x\":-0.999000,\"y\":0.420000}" << p << true;
		}

		{
			VuoPoint2d p;
			p.x = 0;
			p.y = -0.999;
			QTest::newRow("partial point 2") << "{\"y\":-0.999}" << p << false;
		}

		{
			VuoPoint2d p;
			p.x = p.y = 3.40282e+38;
			QTest::newRow("float max") << "{\"x\":3.40282e+38, \"y\":3.40282e+38}" << p << false;
		}

		{
			VuoPoint2d p;
			p.x = p.y = 1.17549e-38;
			QTest::newRow("float min") << "{\"x\":1.17549e-38, \"y\":1.17549e-38}" << p << false;
		}
	}
	void testStringConversion()
	{
		QFETCH(QString, initializer);
		QFETCH(VuoPoint2d, value);
		QFETCH(bool, testStringFromValue);

		VuoPoint2d p = VuoPoint2d_valueFromString(initializer.toUtf8().constData());

		QCOMPARE(p.x,value.x);
		QCOMPARE(p.y,value.y);

		if (testStringFromValue)
			QCOMPARE(VuoPoint2d_stringFromValue(p), initializer.toUtf8().constData());
	}
};

QTEST_APPLESS_MAIN(TestVuoPoint2d)

#include "TestVuoPoint2d.moc"

